@@ -1,8 +1,10 @@
 from libs.logger import initialize_logger
 import sys
 from libs.parse import parse_cl
-from preparedata import prepare
-from makecluster import cluster
+from prepare_data import prepare
+from make_clusters import cluster
+from create_report import report
+from make_predictions import predictions
 from explore_cluster import explore
 from collapse import collapse_fastq
 from stats import stats
@@ -12,7 +14,7 @@ import time
 
 def main(**kwargs):
     kwargs = parse_cl(sys.argv[1:])
-    initialize_logger(kwargs['args'].out, kwargs['args'].debug)
+    initialize_logger(kwargs['args'].out, kwargs['args'].debug, kwargs['args'].print_debug)
     logger = mylog.getLogger(__name__)
     start = time.time()
     if "prepare" in kwargs:
@@ -21,6 +23,12 @@ def main(**kwargs):
     elif "cluster" in kwargs:
         logger.info("Run cluster")
         cluster(kwargs["args"])
+    elif "report" in kwargs:
+        logger.info("Run report")
+        report(kwargs["args"])
+    elif "predict" in kwargs:
+        logger.info("Run predictions")
+        predictions(kwargs["args"])
     elif "explore" in kwargs:
         logger.info("Run explore")
         explore(kwargs["args"])
@@ -30,4 +38,7 @@ def main(**kwargs):
     elif "collapse" in kwargs:
         logger.info("Run collapse")
         collapse_fastq(kwargs["args"])
+    elif "simulator" in kwargs:
+        logger.info("Run simulator")
+        # function to simulator
     logger.info('It took %.3f minutes' % ((time.time()-start)/60))
