@@ -1,20 +1,29 @@
 from collections import defaultdict
 import operator
 import os
+import os.path as op
 import copy
-import time
+from progressbar import ProgressBar
+
+# import time
 import math
-import numpy as np
-import pybedtools
+# import numpy as np
+# import pybedtools
+
+from bcbio.distributed.transaction import file_transaction
+
 import logger as mylog
 from classes import *
 from mystats import up_threshold
-from annotation import _position_in_feature
+from bayes import decide_by_bayes
 import parameters
 
 logger = mylog.getLogger(__name__)
 
 
+REMOVED = 0
+
+
 def get_distance(p1, p2):
         if p1.strand == "+":
                 d = p1.start-p2.start
@@ -24,11 +33,11 @@ def get_distance(p1, p2):
 
 
 def get_ini_str(n):
-        #get space values
+        # get space values
         return "".join(" " for i in range(n))
 
 
-def readbowtie():
+def readbowtie_deprecated():
     ##read results from bowtie...no longer needed
     dict = {}
     f = open("temp.map", 'r')
@@ -50,7 +59,7 @@ def readbowtie():
     return dict
 
 
-def what_is(file):
+def what_is_deprecated(file):
     with open(file, 'r') as handle_in:
         for line in handle_in:
             if not line.startswith("@"):
@@ -155,74 +164,7 @@ def show_seq(clus_obj, index):
     return clus_obj
 
 
-def anncluster(c, clus_obj, db, type_ann):
-    """intersect transcription position with annotation files"""
-    if type_ann == "gtf":
-        id_sa = 1
-        id_ea = 2
-        id_sb = 9
-        id_eb = 10
-        id_id = 3
-        id_idl = 4
-        id_sta = 5
-        id_stb = 12
-        id_tag = 8
-    elif type_ann == "bed":
-        id_sa = 1
-        id_ea = 2
-        id_sb = 7
-        id_eb = 8
-        id_id = 3
-        id_idl = 4
-        id_sta = 5
-        id_stb = 11
-        id_tag = 9
-    ida = 0
-    clus_id = clus_obj.clus
-    loci_id = clus_obj.loci
-    db = os.path.splitext(db)[0]
-    for cols in c.features():
-        id = int(cols[id_id])
-        idl = int(cols[id_idl])
-        if (id in clus_id):
-            clus = clus_id[id]
-            strd = "-"
-            sa = int(cols[id_sa])
-            ea = int(cols[id_ea])
-            sb = int(cols[id_sb])
-            eb = int(cols[id_eb])
-            ida += 1
-            lento5, lento3, strd = _position_in_feature([sa, ea, cols[id_sta]], [sb, eb, cols[id_stb]])
-            if db in loci_id[idl].db_ann:
-                ann = annotation(db, cols[id_tag], strd, lento5, lento3)
-                tdb = loci_id[idl].db_ann[db]
-                tdb.add_db_ann(ida, ann)
-                loci_id[idl].add_db(db, tdb)
-            else:
-                ann = annotation(db, cols[id_tag], strd, lento5, lento3)
-                tdb = dbannotation(1)
-                tdb.add_db_ann(ida, ann)
-                loci_id[idl].add_db(db, tdb)
-            clus_id[id] = clus
-    clus_obj.clus = clus_id
-    clus_obj.loci = loci_id
-    return clus_obj
-
-
-def parse_align_file(file_in):
-    """parse sam files with aligned sequences"""
-    loc_id = 1
-    bedfile_clusters = ""
-    bamfile = pybedtools.BedTool(file_in)
-    bed = pybedtools.BedTool.bam_to_bed(bamfile)
-    for c, start, end, name, q, strand in bed:
-        loc_id += 1
-        bedfile_clusters += "%s\t%s\t%s\t%s\t%s\t%s\n" % \
-                    (c, start, end, name, loc_id, strand)
-    return bedfile_clusters
-
-
-def parse_merge_file(c, current_seq, MIN_SEQ):
+def parse_merge_file_deprecated(c, current_seq, MIN_SEQ):
     """
     Parse the merge file of sequences position to create clusters that will have all
     sequences that shared any position on the genome
@@ -277,7 +219,27 @@ def parse_merge_file(c, current_seq, MIN_SEQ):
     return cluster_info_obj(current_clus, clus_id, current_loci, current_seq)
 
 
-def _get_seqs_from_cluster(seqs, clus_id):
+def add_seqs_position_to_loci_deprecated(fn_bedtools, seqs):
+    """return seqL, with exact position in that loci
+    :param fn_bedtools: pybedtool object
+    :param seqs: object class sequences
+
+    return: dict with locus as key and positions as values"""
+    seqs_pos = defaultdict(defaultdict)
+    for line in fn_bedtools:
+        locus = line[4]
+        name = line[9]
+        pos = [line[6], line[7], line[8]]
+        strand = line[9]
+        start = pos[1] if strand == "+" else pos[2]
+        if start not in seqs_pos[locus]:
+            seqs_pos[locus][start] = []
+        seqs_pos[locus][start].append(name)
+        # print ("{locus} {name} {pos}").format(**locals())
+    return seqs_pos
+
+
+def _get_seqs_from_cluster(seqs, seen):
     """
     Returns the sequences that are already part of the cluster
 
@@ -290,41 +252,76 @@ def _get_seqs_from_cluster(seqs, clus_id):
     """
     already_in = set()
     not_in = []
-    for s in seqs:
-        if s in clus_id:
-            already_in.add(clus_id[s])
-        else:
-            not_in.append(s)
-    return list(already_in), not_in
+
+    already_in = map(seen.get, seqs)
+    # if isinstance(already_in, list):
+    already_in = filter(None, already_in)
+    not_in = set(seqs) - set(seen.keys())
+    # for s in seqs:
+    #    if s in seen:
+    #        already_in.add(seen[s])
+    #    else:
+    #        not_in.append(s)
+    return list(set(already_in)), list(not_in)
 
 
-def reduceloci(clus_obj, min_seq, path):
+def reduceloci(clus_obj,  path):
     """reduce number of loci a cluster has"""
     filtered = {}
     n_cluster = 0
-    current = clus_obj.clus
+    large = 0
+    current = clus_obj.clusid
     logger.info("Number of loci: %s" % len(clus_obj.loci.keys()))
-    for idc in current:
-        logger.debug("_reduceloci: cluster %s" % idc)
-        c = copy.deepcopy(current[idc])
-        n_loci = len(c.loci2seq)
-        if n_loci < 1000:
-            filtered, n_cluster = _iter_loci(c, filtered, n_cluster, min_seq)
-        else:
-            n_cluster += 1
-            filtered[n_cluster] = _add_complete_cluster(n_cluster, c)
+    with ProgressBar(maxval=len(current), redirect_stdout=True) as p:
+        for itern, idmc in enumerate(current):
+            p.update(itern)
+            logger.debug("_reduceloci: cluster %s" % idmc)
+            c = copy.deepcopy(list(current[idmc]))
+
+            n_loci = len(c)
+            if n_loci < 1000:
+                filtered, n_cluster = _iter_loci(c, clus_obj.clus, (clus_obj.loci, clus_obj.seq), filtered, n_cluster)
+            else:
+                large += 1
+                n_cluster += 1
+                _write_cluster(c, clus_obj.clus, clus_obj.loci, n_cluster, path)
+                filtered[n_cluster] = _add_complete_cluster(n_cluster, c, clus_obj.clus)
     clus_obj.clus = filtered
+
+    seqs = 0
+    for idc in filtered:
+        seqs += len(filtered[idc].idmembers)
+    logger.info("seqs in clusters %s" % (seqs))
+
+    logger.info("Clusters too long to be analized: %s" % large)
+    logger.info("Number of clusters removed because low number of reads: %s" % REMOVED)
     return clus_obj
 
 
-def _add_complete_cluster(idx, clus1):
-    logger.debug("Not resolving cluster %s, too many loci. New id %s" % (clus1.id, idx))
-    locilen_sorted = sorted(clus1.locilen.iteritems(), key=operator.itemgetter(1), reverse=True)
+def _write_cluster(metacluster, cluster, loci, idx, path):
+    """
+    For complex meta-clusters, write all the loci for further debug
+    """
+    out_file = op.join(path, 'log', str(idx) + '.bed')
+    with file_transaction(out_file) as out_tx:
+        with open(out_tx, 'w') as out_handle:
+            for idc in metacluster:
+                for idl in cluster[idc].loci2seq:
+                    pos = loci[idl].list()
+                    print >>out_handle, "\t".join(pos[:4] + [str(len(cluster[idc].loci2seq[idl]))] + [pos[-1]])
+
+
+def _add_complete_cluster(idx, meta, clusters):
+    logger.debug("Not resolving cluster %s, too many loci" % (idx))
+    clus = {}
+    [clus.update(clusters[idc].locilen) for idc in meta]
+    locilen_sorted = sorted(clus.iteritems(), key=operator.itemgetter(1), reverse=True)
     maxidl = locilen_sorted[0][0]
     c = cluster(idx)
-    c.add_id_member(clus1.loci2seq[maxidl], maxidl)
+    for idc in meta:
+        c.add_id_member(clusters[idc].idmembers.keys(), maxidl)
     c.id = idx
-    c.toomany = len(locilen_sorted)
+    c.toomany = len(meta)
     return c
 
 
@@ -365,29 +362,31 @@ def _iter_loci_deprecated(c, filtered, n_cluster, min_seq):
     return filtered, n_cluster
 
 
-def _iter_loci(c, filtered, n_cluster, min_seq):
+def _iter_loci(meta, clusters, s2p, filtered, n_cluster):
     """Go through all locus and decide if they are part
     of the same TU or not.
 
     :param idx: int cluster id
+    :param s2p: dict with [loci].coverage[start] = # of sequences there
     :param filtered: dict with clusters object
     :param n_cluster: int cluster id
-    :param min_seq: int min number of sequences inside
-    cluster
 
     :return:
         * filtered: dict of cluster objects
         * n_cluster: int cluster id"""
-    n_loci = len(c.loci2seq)
+    loci = dict(zip(meta, [clusters[idc] for idc in meta]))
+
+    n_loci = len(meta)
     n_loci_prev = n_loci + 1
     cicle = 0
-    #[logger.note("BEFORE %s %s %s" % (c.id, idl, len(c.loci2seq[idl]))) for idl in c.loci2seq]
+    # [logger.note("BEFORE %s %s %s" % (c.id, idl, len(c.loci2seq[idl]))) for idl in c.loci2seq]
     internal_cluster = {}
-    loci = _convert_to_clusters(c)
     if n_loci == 1:
         n_cluster += 1
-        c.id = n_loci
-        filtered[n_cluster] = c
+        filtered[n_cluster] = clusters[meta[0]]
+        filtered[n_cluster].update(id=n_cluster)
+        filtered[n_cluster].set_freq(s2p[1])
+
     while n_loci < n_loci_prev and n_loci != 1:
         n_loci_prev = n_loci
         cicle += 1
@@ -399,18 +398,24 @@ def _iter_loci(c, filtered, n_cluster, min_seq):
         n_loci = len(internal_cluster)
         loci = internal_cluster
         logger.debug("_iter_loci: n_loci %s" % n_loci)
+
     if n_loci > 1:
         n_internal_cluster = sorted(internal_cluster.keys(), reverse=True)[0]
-        internal_cluster = _solve_conflict(internal_cluster, n_internal_cluster)
+        internal_cluster = _solve_conflict(internal_cluster, s2p, n_internal_cluster)
+
     internal_cluster = _clean_cluster(internal_cluster)
+
     for idc in internal_cluster:
         n_cluster += 1
         logger.debug("_iter_loci: add to filtered %s" % n_cluster)
         filtered[n_cluster] = internal_cluster[idc]
         filtered[n_cluster].id = n_cluster
+        filtered[n_cluster].update(id=n_cluster)
+        filtered[n_cluster].set_freq(s2p[1])
     logger.debug("_iter_loci: filtered %s" % filtered.keys())
-    for new_c in internal_cluster.values():
-        [logger.note("%s %s %s %s" % (c.id, new_c.id, idl, len(new_c.loci2seq[idl]))) for idl in new_c.loci2seq]
+
+    # for new_c in internal_cluster.values():
+    #    [logger.note("%s %s %s %s" % (meta, new_c.id, idl, len(new_c.loci2seq[idl]))) for idl in new_c.loci2seq]
     return filtered, n_cluster
 
 
@@ -447,7 +452,7 @@ def _calculate_similarity(c):
     for idc in c:
         set1 = _get_seqs(c[idc])
         [ma.update({(idc, idc2): _common(set1, _get_seqs(c[idc2]))}) for idc2 in c if idc != idc2 and (idc2, idc) not in ma]
-    logger.debug("_calculate_similarity_ %s" % ma)
+    # logger.debug("_calculate_similarity_ %s" % ma)
     return ma
 
 
@@ -455,9 +460,9 @@ def _get_seqs(list_idl):
     """get all sequences in a cluster knowing loci"""
     seqs = set()
     for idl in list_idl.loci2seq:
-        logger.debug("_get_seqs_: loci %s" % idl)
+        # logger.debug("_get_seqs_: loci %s" % idl)
         [seqs.add(s) for s in list_idl.loci2seq[idl]]
-    logger.debug("_get_seqs_: %s" % len(seqs))
+    # logger.debug("_get_seqs_: %s" % len(seqs))
     return seqs
 
 
@@ -516,18 +521,28 @@ def _merge_cluster(old, new):
     logger.debug("_merge_cluster: %s to %s" % (old.id, new.id))
     for idl in old.loci2seq:
         logger.debug("_merge_cluster: add idl %s" % idl)
-        new.loci2seq[idl] = old.loci2seq[idl]
+        # if idl in new.loci2seq:
+        #    new.loci2seq[idl] = list(set(new.loci2seq[idl] + old.loci2seq[idl]))
+        # new.loci2seq[idl] = old.loci2seq[idl]
+        new.add_id_member(old.loci2seq[idl], idl)
     return new
 
 
-def _solve_conflict(list_c, n_cluster):
+def _solve_conflict(list_c, s2p, n_cluster):
     """make sure sequences are counts once.
-    Resolve by most-vote or exclussion"""
+    Resolve by most-vote or exclussion
+
+    :params list_c: dict of objects cluster
+    :param s2p: dict of [loci].coverage = # num of seqs
+    :param n_cluster: number of clusters
+
+    return dict: new set of clusters"""
     logger.debug("_solve_conflict: count once")
+    if parameters.decision_cluster == "bayes":
+        return decide_by_bayes(list_c, s2p)
     loci_similarity = _calculate_similarity(list_c)
     loci_similarity = sorted(loci_similarity.iteritems(), key=operator.itemgetter(1), reverse=True)
     common = sum([score for p, score in loci_similarity])
-    global decision_cluster
     while common:
         n_cluster += 1
         logger.debug("_solve_conflict: ma %s" % loci_similarity)
@@ -546,7 +561,7 @@ def _solve_conflict(list_c, n_cluster):
     return list_c
 
 
-def _split_cluster(c , pairs, n):
+def _split_cluster(c, pairs, n):
     """split cluster by exclussion"""
     old = c[p[0]]
     new = c[p[1]]
@@ -556,16 +571,18 @@ def _split_cluster(c , pairs, n):
         in_common = list(set(common).intersection(old.loci2seq[idl]))
         if len(in_common) > 0:
             logger.debug("_split_cluster: in_common %s with pair 1" % (len(in_common)))
-            new_c.loci2seq[idl] = in_common
+            new_c.add_id_member(in_common, idl)
             old.loci2seq[idl] = list(set(old.loci2seq[idl]) - set(common))
             logger.debug("_split_cluster: len old %s with pair 1" % (len(old.loci2seq)))
     for idl in new.loci2seq:
         in_common = list(set(common).intersection(new.loci2seq[idl]))
         if len(in_common) > 0:
             logger.debug("_split_cluster: in_common %s with pair 2" % (len(in_common)))
-            new_c.loci2seq[idl] = in_common
+            new_c.add_id_member(in_common, idl)
             new.loci2seq[idl] = list(set(new.loci2seq[idl]) - set(common))
             logger.debug("_split_cluster: len old %s with pair 2" % (len(new.loci2seq)))
+    old.update()
+    new.update()
     old.loci2seq = {k: v for k, v in old.loci2seq.iteritems() if len(v) > 0}
     new.loci2seq = {k: v for k, v in new.loci2seq.iteritems() if len(v) > 0}
     c[n] = new
@@ -591,6 +608,8 @@ def _split_cluster_by_most_vote(c, p):
             remove.loci2seq[idl] = list(set(remove.loci2seq[idl]) - set(common))
     keep.loci2seq = {k: v for k, v in keep.loci2seq.iteritems() if len(v) > 0}
     remove.loci2seq = {k: v for k, v in remove.loci2seq.iteritems() if len(v) > 0}
+    keep.update()
+    remove.update()
     c[keep.id] = keep
     c[remove.id] = remove
     return c
@@ -609,11 +628,17 @@ def _add_unseen(loci, clus_seen, n_cluster):
 
 
 def _clean_cluster(list_c):
-    """Remove cluster with less than 10 sequences and
-    loci with size smaller than 60%"""
+    """
+    Remove cluster with less than 10 sequences and
+    loci with size smaller than 60%
+    """
+    global REMOVED
+    init = len(list_c)
     list_c = {k: v for k, v in list_c.iteritems() if len(_get_seqs(v)) > parameters.min_seqs}
     logger.debug("_clean_cluster: number of clusters %s " % len(list_c.keys()))
     list_c = {k: _select_loci(v) for k, v in list_c.iteritems()}
+    end = len(list_c)
+    REMOVED += init - end
     return list_c
 
 
@@ -626,6 +651,8 @@ def _select_loci(c):
     logger.debug("_select_loci: max size %s" % max_size)
     loci_clean = {locus: c.loci2seq[locus] for locus, size in loci_len_sort if size > 0.8 * max_size}
     c.loci2seq = loci_clean
+    removed = list(set(c.idmembers.keys()) - set(_get_seqs(c)))
+    c.add_id_member(removed, loci_len_sort[0][0])
     logger.debug("_select_loci: number of loci %s after cleaning" % len(c.loci2seq.keys()))
     return c
 
@@ -737,33 +764,6 @@ def generate_position_bed(clus_obj):
     return bedaligned
 
 
-def parse_ma_file(in_file):
-    """read seqs.ma file and create dict with
-    sequence object"""
-    name = ""
-    seq_l = {}
-    index = 1
-    total = defaultdict(int)
-    with open(in_file) as handle_in:
-        line = handle_in.readline().strip()
-        cols = line.split("\t")
-        samples = cols[2:]
-        for line in handle_in:
-            line = line.strip()
-            cols = line.split("\t")
-            exp = {}
-            for i in range(len(samples)):
-                exp[samples[i]] = int(cols[i+2])
-                total[samples[i]] += int(cols[i+2])
-            name = cols[0].replace(">", "")
-            index = index+1
-            seq = cols[1]
-            new_s = sequence(seq, exp, index)
-            seq_l[name] = new_s
-    seq_l = _normalize_seqs(seq_l, total)
-    return seq_l
-
-
 def _normalize_seqs(s, t):
     """Normalize to RPM"""
     for ids in s:
